package com.storlead.sales.mapper.customer;

import com.storlead.framework.mybatis.mapper.MyBaseMapper;
import com.storlead.sales.pojo.dto.customer.DataAnalysisDTO;
import com.storlead.sales.pojo.vo.customer.*;
import org.apache.ibatis.annotations.Param;

import java.util.List;

public interface RankingsMapper extends MyBaseMapper {

    List<ClosedBusinessMonthCountVO> selectClosedBusinessByMonth(@Param("query") DataAnalysisDTO dto);

    List<CloseBusinessAllListVO> selectCloseBusinessAllList(@Param("query") DataAnalysisDTO dto);

    List<QuotationSalesListVO> selectQuotationByTotalNumber(@Param("query") DataAnalysisDTO dto);

    List<QuotationTimeListVO> selectQuotationByTime(@Param("query") DataAnalysisDTO dto);

    List<SalesDataAddCountVO> selectBusinessOrderCustomer(@Param("query") DataAnalysisDTO dto);

    List<TransactionDataVo> selectOrderProduct(@Param("query") DataAnalysisDTO dto);

    // 线索转化率
    List<ConversionRateListVo> selectClueConversionRate(@Param("query") DataAnalysisDTO dto);

    // 客户转化率
    List<ConversionRateListVo> selectCustomerConversionRate(@Param("query") DataAnalysisDTO dto);

    // 商机成交率
    List<ConversionRateListVo> selectBusinessConversionRate(@Param("query") DataAnalysisDTO dto);

    // 商机流失率
    List<ConversionRateListVo> selectBusinessLossRate(@Param("query") DataAnalysisDTO dto);

    // 跟进记录
    List<UserFollowUpModeCountVO> selectFollowRecord(@Param("query") DataAnalysisDTO dto);
}
